@@ -72,14 +72,22 @@ look like::
 
         def run_app(self):
             self.framework_app.run()
+            # next statement is optional, for to pass the app exit code to the OS
+            sys.exit(self._exit_code)
 
 Both implementations of these abstract methods will be executed only once
-at app startup. In the first executed method
-:meth:`~MainAppBase.app_init` you could initialize
-the GUI framework and prepare it for the app startup. The :meth:`~MainAppClass.run_app`
+at app startup. Initialize within the first executed method
+:meth:`~MainAppBase.app_init` the GUI framework and
+prepare it for the app startup. The :meth:`~MainAppClass.run_app`
 method will be called from the main module of your app project for to
 start the framework app instance.
 
+If you want ot use the :attr:`~MainAppBase.root_win` and/or
+:attr:`~MainAppBase.root_layout` attributes you can initialize them
+either within :meth:`~MainAppBase.app_init`. At latest they
+should be initialized just before or in the execution of
+:meth:`~MainAppClass.run_app`.
+
 
 key press event dispatching
 ___________________________
@@ -255,7 +263,7 @@ from ae.console import ConsoleApp                   # type: ignore
 from ae.files import FilesRegister, RegisteredFile  # type: ignore
 
 
-__version__ = '0.0.12'
+__version__ = '0.0.13'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -303,6 +311,8 @@ class MainAppBase(ConsoleApp, ABC):
 
         :param console_app_kwargs:  kwargs to be passed to the __init__ method of :class:`~ae.console_app.ConsoleApp`.
         """
+        self._exit_code = 0     # init by stop_app() and passed onto OS by run_app()
+
         self.context_path = list()  # init for Literal type recognition - will be overwritten by setup_app_states()
         super().__init__(**console_app_kwargs)
         self.load_app_states()
@@ -420,13 +430,14 @@ class MainAppBase(ConsoleApp, ABC):
             return self.sound_files(sound_name)
         return None
 
-    def key_press_from_framework(self, modifiers: str, key: str):
+    def key_press_from_framework(self, modifiers: str, key: str) -> bool:
         """ dispatch key press event, coming normalized from the UI framework. """
         event_name = f'on_key_press_of_{modifiers}_{key}'
         en_lower = event_name.lower()
         if not self.call_event(event_name):
             if event_name == en_lower or not self.call_event(en_lower):
-                self.call_event('on_key_press', modifiers, key)
+                return self.call_event('on_key_press', modifiers, key)
+        return True
 
     def load_app_states(self):
         """ load application state for to prepare app.run_app """
@@ -512,6 +523,18 @@ class MainAppBase(ConsoleApp, ABC):
             self.change_app_state(key, val, send_event=False)
             self.set_var(key, val, section=APP_STATE_SECTION_NAME)
 
+    def stop_app(self, exit_code: int = 0):
+        """ quit this application.
+
+        :param exit_code:   optional exit code.
+        """
+        self.dpo(f"stop_app {exit_code}")
+        self._exit_code = exit_code
+
+        if self.root_win:
+            # neither self.framework_app.stop() nor self.framework_app._qapp.exit(exit_code) trigger window closed event
+            self.root_win.close()
+
     def _update_observable_app_states(self, app_state: AppStateType):
         """ update all the observable app states.
 
